/*
 * JerryCrypt (c) - Keep your fro straight.
 *
 * http://WWW.AFRO-PRODUCTIONS.COM
 *
 * Brought to you by the afro productions Yellowish Purple Division
 *
 * This basically uses the jerrycrypt library to encrypt/decrypt files.
 *
 * version information can be found in jcrypt.h
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include "jcrypt.h"

#define METHOD_ENCRYPT	0x01
#define METHOD_DECRYPT	0x02

void fnPrintUsage(char *cmd);
void fnPrintVersion();

int main(int argc, char **argv)
{
	FILE *ifd = stdin, *ofd = stdout;
	int c, method, suppress = 0;
	JCRYPT jc;

	JCRYPT_Initalize(&jc);

	srand(time(NULL));

	jc.GenerateKey(&jc,"AFP"); /* AFP = Afro Productions */
	method = METHOD_ENCRYPT;

	while ((c = getopt(argc,argv,"hvsi:o:k:l:edp")) != EOF)
	{
		switch (c)
		{
			case 'h':
				fnPrintUsage(argv[0]);
			case 'v':
				fnPrintVersion();
			case 'i':
				if ((ifd = fopen(optarg,"r")) == NULL)
				{
					fprintf(stdout,"unable to open %s for reading\n",optarg);
					return 0;
				}
				break;
			case 'o':
				if ((ofd = fopen(optarg,"w")) == NULL)
				{
					fprintf(stdout,"unable to open %s for writing\n",optarg);
					return 0;
				}
				break;
			case 'k':
				jc.GenerateKey(&jc,optarg);
				break;
			case 'e':
				method = METHOD_ENCRYPT;
				break;
			case 'd':
				method = METHOD_DECRYPT;
				break;
			case 'l':
				jc.SetLevel(&jc, atoi(optarg));

				if ((jc.GetLevel(&jc) < 0) || (jc.GetLevel(&jc) > 4))
				{
					fprintf(stdout,"%i is an valid level\n",atoi(optarg));
					return 0;
				}
				break;
			case 's':
				suppress = 1;
				break;
			case 'p':
				jc.PrintKey(&jc, stdout);
				break;
		}
	}

	if ((ifd == stdin) && (!suppress))
		fprintf(stdout,"We'll be using stdin...\n\n");

	if (method == METHOD_ENCRYPT)
		jc.FileEncrypt(&jc,ifd,ofd);
	else
		jc.FileDecrypt(&jc,ifd,ofd);

	return 1;
}

void fnPrintUsage(char *cmd)
{
	fprintf(stdout,"%s\n",J_VERSION);
        fprintf(stdout,"usage: %s [args] [-k key] [-i inputfile] [-o outputfile]\n",cmd);
	fprintf(stdout,"[args]\n");
	fprintf(stdout,"-d           decrypt mode\n");
	fprintf(stdout,"-e           encrypt mode\n");
	fprintf(stdout,"-s           suppress stdin warning message\n");
	fprintf(stdout,"-h           this menu\n");
	fprintf(stdout,"-v           version information\n");
	fprintf(stdout,"-p           print the binary form of your key\n");
	fprintf(stdout,"-l [level]   where level is:\n");
	fprintf(stdout,"             0 - standard encryption (file = same size)\n");
	fprintf(stdout,"             1 - one level above (file = 2 * file size)\n");
	fprintf(stdout,"             2 - two levels above (file = 3 * file size)\n");
	fprintf(stdout,"             3 - three levels above (file = 4 * file size)\n");
	fprintf(stdout,"             level makes it more confusing to \"brute force\"\n");
	fprintf(stdout,"              but you're probably safe is the standard (0)\n");

	exit (0);
}

void fnPrintVersion()
{
        fprintf(stdout,"%s\n",J_VERSION);
        fprintf(stdout,"Brought to you by the afro-productions Yellowish Purple Division.\n");
        fprintf(stdout,"http://www.afro-productions.com\n");

	exit (0);
}
