/*
 * JerryCrypt (c) - Keep your fro straight.
 *
 * http://www.afro-productions.com
 *
 * Created by the afro-productions Yellowish Purple Division.
 *
 * The code below is responsible for implementing and controlling a jerry
 * crypt session (struct jcrypt_st).
 *
 * version information can be found in jcrypt.h
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "jcrypt.h"

int JCRYPT_Initalize(struct jcrypt_st *jc)
{
	if (!jc)
		return -1; /* passing an invalid struct, nice. */

	memset(jc->skey,0,KEY_SIZE);
	
	jc->klen = 0;
	jc->spos = 0;
	jc->kpos = 0;
	jc->kdiff = 1;

	jc->initalized = -1;

	jc->level = L_CASUALLY_PIMP;
	jc->klevel = 0;

	memset(jc->t_strenc,0,MAX_STR_SIZE);
	jc->t_strenclen = 0;

	memset(jc->t_strdec,0,MAX_STR_SIZE);
	jc->t_strdeclen = 0;

	jc->skipped = 0;

	jc->krestart = 1;

	jc->GenerateKey = fnGenKey;
	jc->PrintKey = fnPrintKey;
	jc->StringEncrypt = fnStrEncrypt;
	jc->StringDecrypt = fnStrDecrypt;
	jc->FileEncrypt = fnFileEncrypt;
	jc->FileDecrypt = fnFileDecrypt;
	jc->SetLevel = fnSetLevel;
	jc->GetLevel = fnGetLevel;
	jc->DisableCurrentRestart = fnDisCurrRestart;
	jc->EnableCurrentRestart = fnEnCurrRestart;
	jc->ForceCurrentKeyRestart = fnForceKeyRestart;

	return 1;
}

void fnGenKey(struct jcrypt_st *jc, char *key)
{
	int tret = 0, val = 1;
	int tklen;
	int c;

	if (!_fnCheckInitalized(jc))
		return;

	if (!key)
		return;

	tklen = strlen(key);

	if (tklen == 0)
		return;
	
	memset(jc->skey,0,KEY_SIZE);
	jc->klen = 0;
	jc->kdiff = 0;

	for (c = 0; c < tklen;c++)
	{
		val = 128;

		tret = key[c];

		if (tret < 0)
			tret = abs(tret) + 128;

		while (val != 0)
		{
			if ((tret - val) >= 0)
			{
				jc->skey[jc->klen] = '1';
				tret -= val;

				if ((jc->klen != 0) && (jc->skey[jc->klen-1] == '1'))
					jc->kdiff += 5;
				else
					jc->kdiff += 11;
			}
			else
				jc->skey[jc->klen] = '0';
	
			val /= 2;
	
			jc->klen++;

			if (jc->klen >= KEY_SIZE)
				break;
		}
	}

	if (jc->klen > jc->kdiff)
		jc->spos = jc->klen - jc->kdiff;
	else
		jc->spos = jc->kdiff - jc->klen;

	jc->kpos = 0;

	while (jc->spos >= jc->klen)
	{
		if ((jc->spos - jc->kdiff) < 0)
		{
			if (jc->skey[jc->kpos] == '1')
				jc->spos += jc->kpos * 2;
			else
				jc->spos += jc->kpos;

			jc->kpos++;
			
			if (jc->kpos >= jc->klen)
				jc->kpos = 0;
		}
		else
			jc->spos -= jc->kdiff;
	}

	jc->kpos = jc->spos;
}

void fnPrintKey(struct jcrypt_st *jc, FILE *ofd)
{
	if (!_fnCheckInitalized(jc))
		return;

	fprintf(ofd,"Key=%s\n",jc->skey);
	fflush(ofd);

	return;
}

void fnStrEncrypt(struct jcrypt_st *jc, char *str, int length)
{
	int c, x;
	char *ret;

	if (!_fnCheckInitalized(jc))
		return;	

	if ((length+1) > MAX_STR_SIZE)
		return;

	if (jc->krestart)
	{
		jc->kpos = jc->spos;
		jc->klevel = 0;
	}

	if (jc->t_strenclen == 0)
		memset(jc->t_strenc, 0, sizeof(jc->t_strenc));
	else
		memset(jc->t_strenc,0,jc->t_strenclen);

	jc->t_strenclen = 0;

	for (c = 0; c < length; c++)
	{
		if (c == 0)
			sprintf(jc->t_strenc,"%s",_fnEncryptLetter(jc,str[c]));
		else
		{
			ret = _fnEncryptLetter(jc,str[c]);

			if ((jc->t_strenclen+jc->level+2) >= MAX_STR_SIZE)
				break;

			for (x = 0; x <= jc->level; x++)
				jc->t_strenc[jc->t_strenclen+x] = ret[x];
		}

		jc->t_strenclen += (jc->level+1);
	}

	return;
}

void fnStrDecrypt(struct jcrypt_st *jc, char *str, int length)
{
	int c, let;

	if (!_fnCheckInitalized(jc))
		return;

	if ((length+1) > MAX_STR_SIZE)
		return;

	if (jc->krestart)
	{
		jc->kpos = jc->spos;
		jc->klevel = 0;
	}

	memset(jc->t_strdec,0,jc->t_strdeclen);
	jc->t_strdeclen = 0;

	for (c = 0; c < length; c++)
	{
		if (jc->t_strdeclen+1 >= MAX_STR_SIZE)
			break;

		let = _fnDecryptLetter(jc,str[c]);

		if (!jc->skipped)
			jc->t_strdec[jc->t_strdeclen++] = let;
	}

	return;
}


void fnFileEncrypt(struct jcrypt_st *jc, FILE *ifd, FILE *ofd)
{
	char tbuf[2] = { 0, 0 };
	char *ret;
	int c;

	if (!_fnCheckInitalized(jc))
		return;

	jc->kpos = jc->spos;
	jc->klevel = 0;

	while (fread(tbuf,1,1,ifd))
	{
		ret = _fnEncryptLetter(jc,tbuf[0]);

		for (c = 0; c <= jc->level; c++)
		{
			fprintf(ofd,"%c",ret[c]);

			if (c == jc->level)
				break;
		}

		fflush(ofd);
	}		

	return;
}


void fnFileDecrypt(struct jcrypt_st *jc, FILE *ifd, FILE *ofd)
{
	char tbuf[2] = { 0, 0 };
	int let;

	if (!_fnCheckInitalized(jc))
		return;

	jc->kpos = jc->spos;
	jc->klevel = 0;

	while (fread(tbuf,1,1,ifd))
	{
		let = _fnDecryptLetter(jc,tbuf[0]);

		if (!jc->skipped)
			fprintf(ofd,"%c",let);

		fflush(ofd);
	}

	return;
}

char *_fnEncryptLetter(struct jcrypt_st *jc, int letter)
{
	static char ret[6];

	memset(ret,0,jc->level+2);

	for (jc->klevel = 0; jc->klevel <= jc->level; jc->klevel++)
	{
		if (jc->kpos >= jc->klen)
			jc->kpos = 0;

		if (jc->klevel == jc->level)
		{
			if (jc->skey[jc->kpos] == '1')
				ret[jc->klevel] = letter-(jc->kpos+1+jc->kdiff);
			else
				ret[jc->klevel] = letter+(jc->kpos+1+jc->kdiff);

			jc->kpos++;

			break;
		}
		else
			ret[jc->klevel] = rand()%253+1;
		
		jc->kpos++;
	}

	return ret;
}

int _fnDecryptLetter(struct jcrypt_st *jc, int letter)
{
	int retlet = 0;

	if (jc->kpos >= jc->klen)
		jc->kpos = 0;

        if (jc->klevel > jc->level)
                jc->klevel = 0;

        if (jc->klevel == jc->level)
        {
                if (jc->skey[jc->kpos] == '1')
                        retlet = letter+(jc->kpos+1+jc->kdiff);
                else
                        retlet = letter-(jc->kpos+1+jc->kdiff);

		jc->skipped = 0;
        }
	else
		jc->skipped = 1;
        
        jc->kpos++;
	jc->klevel++;	

	return retlet;
}

int _fnCheckInitalized(struct jcrypt_st *jc)
{
	if (!jc)
		return 0;

	if (jc->initalized != -1)
		return 0;

	return 1;
}

void fnSetLevel(struct jcrypt_st *jc, int level)
{
	if (!_fnCheckInitalized(jc))
		return;

	if ((level < 0) || (level > L_SHAFTLIKE_PIMP))
		return;

	jc->level = level;

	return;
}

int fnGetLevel(struct jcrypt_st *jc)
{
	if (!_fnCheckInitalized(jc))
		return -1;

	return jc->level;
}

void fnDisCurrRestart(struct jcrypt_st *jc)
{
	if (!_fnCheckInitalized(jc))
		return;

	jc->krestart = 0;

	return;
}

void fnEnCurrRestart(struct jcrypt_st *jc)
{
	if (!_fnCheckInitalized(jc))
		return;

	jc->krestart = 1;

	return;
}

void fnForceKeyRestart(struct jcrypt_st *jc)
{
	if (!_fnCheckInitalized(jc))
		return;

	jc->kpos = jc->spos;

	return;
}

